/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Copyrighted 2002-2012 George Waldon
 */

package org.starbeans.modules.corelite.applications;

import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.filechooser.FileSystemView;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.util.Lookup;

/**
 *
 * @author gwaldon
 */
public class ApplicationNode extends DataNode {
    
    private static Map<String,Image> imap = new HashMap<String,Image>();

    public ApplicationNode (DataObject obj, Children ch, Lookup lookup) {
        super(obj,ch,lookup);
    }

     @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    // See caching icons in javanode
    @Override
    public Image getIcon(int type) {
        DataObject obj = getDataObject();
        FileObject fObj = obj.getPrimaryFile();
        String ext = fObj.getExt();
        if(ext!=null && !ext.isEmpty()) {
            Image img = getImageFromFile(fObj);
            if(img!=null)
                return img;
        }
        return super.getIcon(type);
    }

    private synchronized Image getImageFromFile(FileObject fObj) {
        String ext = fObj.getExt();
        if(ext==null || ext.isEmpty())
            return null;
        if(imap.containsKey(ext)) {
            return imap.get(ext);
        }

        File file;
        Icon systemIcon = null;
        try {
            file = File.createTempFile("icon", "."+fObj.getExt());
            FileSystemView view = FileSystemView.getFileSystemView();
            systemIcon = FileSystemView.getFileSystemView().getSystemIcon(file);
            file.delete();
        } catch (IOException ex) {
            return null;
        }
        if(systemIcon!=null) {
            Image img = iconToImage(systemIcon);
            imap.put(ext, img);
            return img;
        }
        return null;
    }
    
    private Image iconToImage(Icon icon) {
        if (icon instanceof ImageIcon) {
            return ((ImageIcon) icon).getImage();
        } else {
            int w = icon.getIconWidth();
            int h = icon.getIconHeight();
            GraphicsEnvironment ge =
                    GraphicsEnvironment.getLocalGraphicsEnvironment();
            GraphicsDevice gd = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gd.getDefaultConfiguration();
            BufferedImage image = gc.createCompatibleImage(w, h);
            Graphics2D g = image.createGraphics();
            icon.paintIcon(null, g, 0, 0);
            g.dispose();
            return image;
        }
    }

}
